#-----------------------------------------------------------------------------
# Name:        languagectrls.py
# Purpose:     
#
# Author:      Riaan Booysen
#
# Created:     2006
# RCS-ID:      $Id$
# Copyright:   (c) 2006 Riaan Booysen
# License:     wxPython
#-----------------------------------------------------------------------------
""" ListCtrl and functions to display languages and the flags of their countries
"""
import wx

from wx.lib.art import flagart

langIdCountryMap = {
    # generated from wx.Locale info and locale.windows_locale
    wx.wxLANGUAGE_AFRIKAANS: 'ZA',
    wx.wxLANGUAGE_ALBANIAN: 'AL',
    wx.wxLANGUAGE_ARABIC_ALGERIA: 'DZ',
    wx.wxLANGUAGE_ARABIC_BAHRAIN: 'BH',
    wx.wxLANGUAGE_ARABIC_EGYPT: 'EG',
    wx.wxLANGUAGE_ARABIC_IRAQ: 'IQ',
    wx.wxLANGUAGE_ARABIC_JORDAN: 'JO',
    wx.wxLANGUAGE_ARABIC_KUWAIT: 'KW',
    wx.wxLANGUAGE_ARABIC_LEBANON: 'LB',
    wx.wxLANGUAGE_ARABIC_LIBYA: 'LY',
    wx.wxLANGUAGE_ARABIC_MOROCCO: 'MA',
    wx.wxLANGUAGE_ARABIC_OMAN: 'OM',
    wx.wxLANGUAGE_ARABIC_QATAR: 'QA',
    wx.wxLANGUAGE_ARABIC_SAUDI_ARABIA: 'SA',
    wx.wxLANGUAGE_ARABIC_SUDAN: 'SD',
    wx.wxLANGUAGE_ARABIC_SYRIA: 'SY',
    wx.wxLANGUAGE_ARABIC_TUNISIA: 'TN',
    wx.wxLANGUAGE_ARABIC_UAE: 'AE',
    wx.wxLANGUAGE_ARABIC_YEMEN: 'YE',
    wx.wxLANGUAGE_ARMENIAN: 'AM',
    wx.wxLANGUAGE_AZERI: 'AZ',
    wx.wxLANGUAGE_AZERI_CYRILLIC: 'AZ',
    wx.wxLANGUAGE_AZERI_LATIN: 'AZ',
    wx.wxLANGUAGE_BASQUE: 'ES',
    wx.wxLANGUAGE_BELARUSIAN: 'BY',
    wx.wxLANGUAGE_BENGALI: 'IN',
    wx.wxLANGUAGE_BRETON: 'FR',
    wx.wxLANGUAGE_BULGARIAN: 'BG',
    wx.wxLANGUAGE_CATALAN: 'ES',
    wx.wxLANGUAGE_CHINESE: 'CN',
    wx.wxLANGUAGE_CHINESE_HONGKONG: 'HK',
    wx.wxLANGUAGE_CHINESE_MACAU: 'MO',
    wx.wxLANGUAGE_CHINESE_SIMPLIFIED: 'CN',
    wx.wxLANGUAGE_CHINESE_SINGAPORE: 'SG',
    wx.wxLANGUAGE_CHINESE_TAIWAN: 'TW',
    wx.wxLANGUAGE_CHINESE_TRADITIONAL: 'CN',
    wx.wxLANGUAGE_CROATIAN: 'HR',
    wx.wxLANGUAGE_CZECH: 'CZ',
    wx.wxLANGUAGE_DANISH: 'DK',
#    wx.wxLANGUAGE_DEFAULT: 'ZA',
    wx.wxLANGUAGE_DUTCH: 'NL',
    wx.wxLANGUAGE_DUTCH_BELGIAN: 'BE',
    wx.wxLANGUAGE_ENGLISH: 'GB',
    wx.wxLANGUAGE_ENGLISH_AUSTRALIA: 'AU',
    wx.wxLANGUAGE_ENGLISH_BELIZE: 'BZ',
    wx.wxLANGUAGE_ENGLISH_BOTSWANA: 'BW',
    wx.wxLANGUAGE_ENGLISH_CANADA: 'CA',
    wx.wxLANGUAGE_ENGLISH_CARIBBEAN: 'CB',
    wx.wxLANGUAGE_ENGLISH_DENMARK: 'DK',
    wx.wxLANGUAGE_ENGLISH_EIRE: 'IE',
    wx.wxLANGUAGE_ENGLISH_JAMAICA: 'JM',
    wx.wxLANGUAGE_ENGLISH_NEW_ZEALAND: 'NZ',
    wx.wxLANGUAGE_ENGLISH_PHILIPPINES: 'PH',
    wx.wxLANGUAGE_ENGLISH_SOUTH_AFRICA: 'ZA',
    wx.wxLANGUAGE_ENGLISH_TRINIDAD: 'TT',
    wx.wxLANGUAGE_ENGLISH_UK: 'GB',
    wx.wxLANGUAGE_ENGLISH_US: 'US',
    wx.wxLANGUAGE_ENGLISH_ZIMBABWE: 'ZW',
    wx.wxLANGUAGE_ESTONIAN: 'EE',
    wx.wxLANGUAGE_FAEROESE: 'FO',
    wx.wxLANGUAGE_FARSI: 'IR',
    wx.wxLANGUAGE_FINNISH: 'FI',
    wx.wxLANGUAGE_FRENCH: 'FR',
    wx.wxLANGUAGE_FRENCH_BELGIAN: 'BE',
    wx.wxLANGUAGE_FRENCH_CANADIAN: 'CA',
    wx.wxLANGUAGE_FRENCH_LUXEMBOURG: 'LU',
    wx.wxLANGUAGE_FRENCH_MONACO: 'MC',
    wx.wxLANGUAGE_FRENCH_SWISS: 'CH',
    wx.wxLANGUAGE_FRISIAN: 'NL',
    wx.wxLANGUAGE_GALICIAN: 'ES',
    wx.wxLANGUAGE_GEORGIAN: 'GE',
    wx.wxLANGUAGE_GERMAN: 'DE',
    wx.wxLANGUAGE_GERMAN_AUSTRIAN: 'AT',
    wx.wxLANGUAGE_GERMAN_BELGIUM: 'BE',
    wx.wxLANGUAGE_GERMAN_LIECHTENSTEIN: 'LI',
    wx.wxLANGUAGE_GERMAN_LUXEMBOURG: 'LU',
    wx.wxLANGUAGE_GERMAN_SWISS: 'CH',
    wx.wxLANGUAGE_GREEK: 'GR',
    wx.wxLANGUAGE_GREENLANDIC: 'GL',
    wx.wxLANGUAGE_GUJARATI: 'IN',
    wx.wxLANGUAGE_HEBREW: 'IL',
    wx.wxLANGUAGE_HINDI: 'IN',
    wx.wxLANGUAGE_HUNGARIAN: 'HU',
    wx.wxLANGUAGE_ICELANDIC: 'IS',
    wx.wxLANGUAGE_INDONESIAN: 'ID',
    wx.wxLANGUAGE_INUKTITUT: 'CA',
    wx.wxLANGUAGE_IRISH: 'IE',
    wx.wxLANGUAGE_ITALIAN: 'IT',
    wx.wxLANGUAGE_ITALIAN_SWISS: 'CH',
    wx.wxLANGUAGE_JAPANESE: 'JP',
    wx.wxLANGUAGE_KANNADA: 'IN',
    wx.wxLANGUAGE_KASHMIRI_INDIA: 'IN',
    wx.wxLANGUAGE_KAZAKH: 'KZ',
    wx.wxLANGUAGE_KERNEWEK: 'GB',
    wx.wxLANGUAGE_KIRGHIZ: 'KG',
    wx.wxLANGUAGE_KOREAN: 'KR',
    wx.wxLANGUAGE_LATVIAN: 'LV',
    wx.wxLANGUAGE_LITHUANIAN: 'LT',
    wx.wxLANGUAGE_MACEDONIAN: 'MK',
    wx.wxLANGUAGE_MALAY: 'MY',
    wx.wxLANGUAGE_MALAYALAM: 'IN',
    wx.wxLANGUAGE_MALAY_BRUNEI_DARUSSALAM: 'BN',
    wx.wxLANGUAGE_MALAY_MALAYSIA: 'MY',
    wx.wxLANGUAGE_MALTESE: 'MT',
    wx.wxLANGUAGE_MAORI: 'NZ',
    wx.wxLANGUAGE_MARATHI: 'IN',
    wx.wxLANGUAGE_MONGOLIAN: 'MN',
    wx.wxLANGUAGE_NEPALI: 'NP',
    wx.wxLANGUAGE_NEPALI_INDIA: 'IN',
    wx.wxLANGUAGE_NORWEGIAN_BOKMAL: 'NO',
    wx.wxLANGUAGE_NORWEGIAN_NYNORSK: 'NO',
    wx.wxLANGUAGE_OCCITAN: 'FR',
    wx.wxLANGUAGE_ORIYA: 'IN',
    wx.wxLANGUAGE_PASHTO: 'AF',
    wx.wxLANGUAGE_POLISH: 'PL',
    wx.wxLANGUAGE_PORTUGUESE: 'PT',
    wx.wxLANGUAGE_PORTUGUESE_BRAZILIAN: 'BR',
    wx.wxLANGUAGE_PUNJABI: 'IN',
    wx.wxLANGUAGE_RHAETO_ROMANCE: 'CH',
    wx.wxLANGUAGE_ROMANIAN: 'RO',
    wx.wxLANGUAGE_RUSSIAN: 'RU',
    wx.wxLANGUAGE_RUSSIAN_UKRAINE: 'UA',
    wx.wxLANGUAGE_SANSKRIT: 'IN',
    wx.wxLANGUAGE_SERBIAN_CYRILLIC: 'YU',
    wx.wxLANGUAGE_SERBIAN_LATIN: 'YU',
    wx.wxLANGUAGE_SETSWANA: 'ZA',
    wx.wxLANGUAGE_SLOVAK: 'SK',
    wx.wxLANGUAGE_SLOVENIAN: 'SI',
    wx.wxLANGUAGE_SPANISH: 'ES',
    wx.wxLANGUAGE_SPANISH_ARGENTINA: 'AR',
    wx.wxLANGUAGE_SPANISH_BOLIVIA: 'BO',
    wx.wxLANGUAGE_SPANISH_CHILE: 'CL',
    wx.wxLANGUAGE_SPANISH_COLOMBIA: 'CO',
    wx.wxLANGUAGE_SPANISH_COSTA_RICA: 'CR',
    wx.wxLANGUAGE_SPANISH_DOMINICAN_REPUBLIC: 'DO',
    wx.wxLANGUAGE_SPANISH_ECUADOR: 'EC',
    wx.wxLANGUAGE_SPANISH_EL_SALVADOR: 'SV',
    wx.wxLANGUAGE_SPANISH_GUATEMALA: 'GT',
    wx.wxLANGUAGE_SPANISH_HONDURAS: 'HN',
    wx.wxLANGUAGE_SPANISH_MEXICAN: 'MX',
    wx.wxLANGUAGE_SPANISH_MODERN: 'ES',
    wx.wxLANGUAGE_SPANISH_NICARAGUA: 'NI',
    wx.wxLANGUAGE_SPANISH_PANAMA: 'PA',
    wx.wxLANGUAGE_SPANISH_PARAGUAY: 'PY',
    wx.wxLANGUAGE_SPANISH_PERU: 'PE',
    wx.wxLANGUAGE_SPANISH_PUERTO_RICO: 'PR',
    wx.wxLANGUAGE_SPANISH_URUGUAY: 'UY',
    wx.wxLANGUAGE_SPANISH_US: 'US',
    wx.wxLANGUAGE_SPANISH_VENEZUELA: 'VE',
    wx.wxLANGUAGE_SWAHILI: 'KE',
    wx.wxLANGUAGE_SWEDISH: 'SE',
    wx.wxLANGUAGE_SWEDISH_FINLAND: 'FI',
    wx.wxLANGUAGE_TAGALOG: 'PH',
    wx.wxLANGUAGE_TAMIL: 'IN',
    wx.wxLANGUAGE_TATAR: 'RU',
    wx.wxLANGUAGE_TELUGU: 'IN',
    wx.wxLANGUAGE_THAI: 'TH',
    wx.wxLANGUAGE_TURKISH: 'TR',
    wx.wxLANGUAGE_UKRAINIAN: 'UA',
    wx.wxLANGUAGE_URDU: 'PK',
    wx.wxLANGUAGE_URDU_INDIA: 'IN',
    wx.wxLANGUAGE_URDU_PAKISTAN: 'PK',
    wx.wxLANGUAGE_UZBEK: 'UZ',
    wx.wxLANGUAGE_UZBEK_CYRILLIC: 'UZ',
    wx.wxLANGUAGE_UZBEK_LATIN: 'UZ',
    wx.wxLANGUAGE_VIETNAMESE: 'VN',
    wx.wxLANGUAGE_WELSH: 'GB',
    wx.wxLANGUAGE_XHOSA: 'ZA',
    wx.wxLANGUAGE_ZULU: 'ZA',
    # manually defined language/country mapping
    wx.wxLANGUAGE_ABKHAZIAN: 'GE',
    wx.wxLANGUAGE_AFAR: 'ET',
    wx.wxLANGUAGE_AMHARIC: 'ET',
    wx.wxLANGUAGE_ASSAMESE: 'IN',
    wx.wxLANGUAGE_AYMARA: 'BO',
    wx.wxLANGUAGE_ARABIC: 'SA', 
    wx.wxLANGUAGE_BASHKIR: 'RU',
    wx.wxLANGUAGE_BHUTANI: 'BT',
    wx.wxLANGUAGE_BIHARI: 'IN',
    wx.wxLANGUAGE_BISLAMA: 'VU',
    wx.wxLANGUAGE_BURMESE: 'MM',
    wx.wxLANGUAGE_CAMBODIAN: 'KH',
    wx.wxLANGUAGE_CORSICAN: 'FR',
    wx.wxLANGUAGE_ESPERANTO: 'ESPERANTO',
    wx.wxLANGUAGE_FIJI: 'FJ',
    wx.wxLANGUAGE_GUARANI: 'PY',
    wx.wxLANGUAGE_HAUSA: 'NG',
    wx.wxLANGUAGE_INTERLINGUA: 'US', 
    wx.wxLANGUAGE_INTERLINGUE: 'US',
    wx.wxLANGUAGE_INUPIAK: 'US',
    wx.wxLANGUAGE_JAVANESE: 'IN',
    wx.wxLANGUAGE_KASHMIRI: 'IN',
    wx.wxLANGUAGE_KINYARWANDA: 'RW',
    wx.wxLANGUAGE_KIRUNDI: 'BI',
    wx.wxLANGUAGE_KONKANI: 'IN',
    wx.wxLANGUAGE_KURDISH: 'IQ',
    wx.wxLANGUAGE_LAOTHIAN: 'LA',
    wx.wxLANGUAGE_LATIN: 'VA',
    wx.wxLANGUAGE_LINGALA: 'CD',
    wx.wxLANGUAGE_MALAGASY: 'MG',
    wx.wxLANGUAGE_MANIPURI: 'IN',
    wx.wxLANGUAGE_MOLDAVIAN: 'MD',
    wx.wxLANGUAGE_NAURU: 'NR',
    wx.wxLANGUAGE_OROMO: 'ET',
    wx.wxLANGUAGE_QUECHUA: 'BO',
    wx.wxLANGUAGE_SAMOAN: 'WS',
    wx.wxLANGUAGE_SANGHO: 'CF',
    wx.wxLANGUAGE_SCOTS_GAELIC: 'GB',
    wx.wxLANGUAGE_SERBO_CROATIAN: 'HR',
    wx.wxLANGUAGE_SESOTHO: 'ZA',
    wx.wxLANGUAGE_SHONA: 'ZW',
    wx.wxLANGUAGE_SINDHI: 'PK',
    wx.wxLANGUAGE_SINHALESE: 'IN',
    wx.wxLANGUAGE_SISWATI: 'SZ',
    wx.wxLANGUAGE_SOMALI: 'SB',
    wx.wxLANGUAGE_SUNDANESE: 'SD',
    wx.wxLANGUAGE_TAJIK: 'TJ',
    wx.wxLANGUAGE_TIBETAN: 'CN',
    wx.wxLANGUAGE_TIGRINYA: 'ET',
    wx.wxLANGUAGE_TONGA: 'TO',
    wx.wxLANGUAGE_TSONGA: 'MZ',
    wx.wxLANGUAGE_TURKMEN: 'TM',
    wx.wxLANGUAGE_TWI: 'GH',
    wx.wxLANGUAGE_UIGHUR: 'CN',
    wx.wxLANGUAGE_VOLAPUK: 'VOLAPUK',
    wx.wxLANGUAGE_WOLOF: 'SN',
    wx.wxLANGUAGE_YIDDISH: 'IL',
    wx.wxLANGUAGE_YORUBA: 'NG',
    wx.wxLANGUAGE_ZHUANG: 'CN',
}

LC_AVAILABLE, LC_ALL, LC_ONLY = 1, 2, 4


# wx.wxLANGUAGE_SERBIAN gives an error for me
_wxLangIds = [n for n in dir(wx) if n.startswith('LANGUAGE_')]
for _l in ('LANGUAGE_UNKNOWN', 'LANGUAGE_USER_DEFINED', 'LANGUAGE_SERBIAN'):
    if _l in _wxLangIds:
        _wxLangIds.remove(_l)


def CreateLanguagesResourceLists(filter=LC_AVAILABLE, only=()):
    """ Returns a tuple of (bitmaps, language descriptions, language ids) """
    icons = wx.ImageList(16, 11)
    names = []
    langs = []

    langIdNameMap = BuildLanguageCountryMapping()

    wxLangIds = []
    for li in _wxLangIds:
         wxLI = getattr(wx, li)
         try:
             if (filter == LC_ONLY and wxLI in only) or \
                (filter == LC_AVAILABLE and wx.Locale.IsAvailable(wxLI)) or \
                (filter == LC_ALL):
                 wxLangIds.append(wxLI)
         except wx.PyAssertionError: 
             # invalid language assertions
             pass
         except AttributeError:
             # wx 2.6
             wxLangIds.append(wxLI)
                 
    langCodes = [(langIdNameMap[wxLangId], wxLangId) 
                 for wxLangId in wxLangIds 
                 if wxLangId in langIdNameMap]
    
    for lc, wxli in langCodes:
        l, cnt = lc.split('_')
        
        if cnt in flagart.catalog:
            bmp = flagart.catalog[cnt].getBitmap()
        else:
            bmp = flagart.catalog['BLANK'].getBitmap()

        icons.Add(bmp)
        name = wx.Locale.GetLanguageName(wxli)
        if wxli == wx.wxLANGUAGE_DEFAULT:
            #print cnt, name, lc, wxli
            name = 'Default: '+name
        
        names.append(name)
        langs.append(wxli)
    
    return icons, names, langs


def GetLanguageFlag(lang):
    """ Returns a bitmap of the flag for the country of the language id """
    langIdNameMap = BuildLanguageCountryMapping()
    if lang in langIdNameMap:
        cnt = langIdNameMap[lang].split('_')[1]
        if cnt in flagart.catalog:
            return flagart.catalog[cnt].getBitmap()
    return flagart.catalog['BLANK'].getBitmap()


def BuildLanguageCountryMapping():
    """ Builds a mapping of language ids to LANG_COUNTRY codes """
    res = {}
    for name in _wxLangIds:
        n = 'wx.'+name
        wn = getattr(wx, name)
        
        li = wx.Locale.GetLanguageInfo(wn)
        if li: 
            code = li.CanonicalName

            if wn in langIdCountryMap:
                # override, drop country
                if '_' in code:
                    code = code.split('_')[0]
                code += '_'+langIdCountryMap[wn]

            # map unhandled to blank images
            elif '_' not in code:
                code += '_BLANK'
    
            res[wn] = code
    return res

def GetWxIdentifierForLanguage(lang):
    """ Returns the language id as a string """ 
    for n in dir(wx):
        if n.startswith('LANGUAGE_') and getattr(wx, n) == lang:
            return n
    raise Exception, 'Language %s not found'%lang


#-------------------------------------------------------------------------------

class LanguageListCtrl(wx.ListCtrl):
    """ wx.ListCtrl derived control that displays languages and flags """
    def __init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=wx.LC_REPORT | wx.LC_NO_HEADER | wx.LC_SINGLE_SEL, 
                 filter=LC_AVAILABLE, only=(), select=None, name='languagelistctrl'):

        wx.ListCtrl.__init__(self, parent, id, pos, size, style, name=name)
        
        self.SetUpFilter(filter, only)
        self.Language = select

    def SetUpFilter(self, filter=LC_AVAILABLE, only=()):
        """ Filters the languages displayed in the control """
        lang = self.GetLanguage()
        
        self.filter, self.only = filter, only
        self.icons, self.choices, self.langs = CreateLanguagesResourceLists(filter, only)
        
        self.AssignImageList(self.icons, wx.IMAGE_LIST_SMALL)
        
        self.ClearAll()
        self.InsertColumn(0, '', width=175)
        for i in range(len(self.choices)):
            self.InsertImageStringItem(i, self.choices[i], i)
        
        self.SetLanguage(lang)

    def GetLanguage(self):
        """ Returns the language id for the currently selected language in the control """
        idx = self.GetFirstSelected()
        if idx != -1:
            return self.langs[idx]
        else:
            None
    
    def SetLanguage(self, lang):
        """ Selects the given language ids item in the control """
        if lang is not None:
            if lang in self.langs:
                idx = self.langs.index(lang)
                self.Select(idx)
                self.Focus(idx)

    Language = property(GetLanguage, SetLanguage, doc="See `GetLanguage` and `SetLanguage`") 
    
#-------------------------------------------------------------------------------


if __name__ == '__main__':
    a = wx.App()
    
    print GetLanguageFlag(wx.wxLANGUAGE_AFRIKAANS)
    
    f=wx.Frame(None, -1)
    f.p=wx.Panel(f, -1)
    s=wx.BoxSizer(wx.VERTICAL)
    f.p.SetSizer(s)
    try:
        f.lc=LanguageChoice(f.p, pos = (220, 10), size = (200, 25))
        s.Add(f.lc, 0, wx.GROW)
    except:
        pass
    f.llc=LanguageListCtrl(f.p, pos = (10, 10), size = (200, 200), 
          filter=LC_ONLY, 
          only=(wx.wxLANGUAGE_AFRIKAANS, wx.wxLANGUAGE_ENGLISH, 
            wx.wxLANGUAGE_FRENCH, wx.wxLANGUAGE_GERMAN, wx.wxLANGUAGE_ITALIAN, 
            wx.wxLANGUAGE_PORTUGUESE_BRAZILIAN, wx.wxLANGUAGE_SPANISH), 
          select=wx.wxLANGUAGE_ENGLISH)
##            filter=LC_ALL)
    s.Add(f.llc, 1, wx.GROW)
    f.Show()
    
    a.MainLoop()
